﻿using System;
using System.Windows.Forms;
using SharpDX;
using SharpDX.Direct2D1;
using SharpDX.Direct3D;
using SharpDX.Direct3D11;
using SharpDX.DXGI;
using D2D1AlphaMode = SharpDX.Direct2D1.AlphaMode;
using D2D1Device = SharpDX.Direct2D1.Device;
using D3D11Device = SharpDX.Direct3D11.Device;
using D3D11Device1 = SharpDX.Direct3D11.Device;
using DeviceContext = SharpDX.Direct2D1.DeviceContext;
using DXGIDevice = SharpDX.DXGI.Device;
using DXGIFactory = SharpDX.DXGI.Factory;
using Factory = SharpDX.Direct2D1.Factory;

namespace Direct2DControl
{
	/// <summary>
	/// 在 WinForm 中使用 Direct2D Demo。
	/// 文章链接：http://www.cnblogs.com/cyjb/p/Direct2DControl.html
	/// by CYJB http://www.cnblogs.com/cyjb/。
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// DXGI 格式。
		/// </summary>
		private static readonly Format Format = Format.B8G8R8A8_UNorm;
		/// <summary>
		/// Direct2D 的像素格式。
		/// </summary>
		public static readonly PixelFormat D2PixelFormat =
			new PixelFormat(Format, D2D1AlphaMode.Premultiplied);
		/// <summary>
		/// 位图的属性。
		/// </summary>
		private static BitmapProperties1 BitmapProps1 = new BitmapProperties1(D2PixelFormat, 96, 96, BitmapOptions.Target);
		/// <summary>
		/// 构造函数。
		/// </summary>
		public MainForm()
		{
			InitializeComponent();
			InitHwndRenderTarget();
			InitDeviceContext();
		}

		#region HwndRenderTarget

		/// <summary>
		/// Hwnd 渲染目标。
		/// </summary>
		private WindowRenderTarget hwndRenderTarget;
		/// <summary>
		/// Hwnd 画刷。
		/// </summary>
		private Brush hwndBrush;
		/// <summary>
		/// 初始化 HwndRenderTarget。
		/// </summary>
		private void InitHwndRenderTarget()
		{
			// 创建 Direct2D 单线程工厂。
			Factory factory = new Factory(FactoryType.SingleThreaded);
			// 渲染参数。
			RenderTargetProperties renderProps = new RenderTargetProperties
			{
				PixelFormat = D2PixelFormat,
				Usage = RenderTargetUsage.None,
				Type = RenderTargetType.Default
			};
			// 渲染目标属性。
			HwndRenderTargetProperties hwndProps = new HwndRenderTargetProperties()
			{
				// 承载控件的句柄。
				Hwnd = hwndRenderControl.Handle,
				// 控件的尺寸。
				PixelSize = new Size2(hwndRenderControl.ClientSize.Width, hwndRenderControl.ClientSize.Height),
				PresentOptions = PresentOptions.None
			};
			// 渲染目标。
			hwndRenderTarget = new WindowRenderTarget(factory, renderProps, hwndProps)
			{
				AntialiasMode = AntialiasMode.PerPrimitive
			};
			// 初始化画刷资源。
			hwndBrush = new SolidColorBrush(hwndRenderTarget, Color.Black);
		}
		/// <summary>
		/// 绘制控件的事件。
		/// </summary>
		private void hwndRenderControl_Paint(object sender, PaintEventArgs e)
		{
			hwndRenderTarget.BeginDraw();
			hwndRenderTarget.Clear(Color.White);
			hwndRenderTarget.DrawRectangle(new RectangleF(10, 10, 200, 200), hwndBrush);
			hwndRenderTarget.EndDraw();
		}
		/// <summary>
		/// 控件大小被改变的事件。
		/// </summary>
		private void hwndRenderControl_SizeChanged(object sender, EventArgs e)
		{
			// 在控件大小被改变的时候必须改变渲染目标的大小，
			// 否则会导致绘制结果被拉伸，引起失真。
			hwndRenderTarget.Resize(new Size2(hwndRenderControl.ClientSize.Width, hwndRenderControl.ClientSize.Height));
		}

		#endregion // HwndRenderTarget

		#region DeviceContext

		/// <summary>
		/// Direct2D 设备上下文。
		/// </summary>
		private DeviceContext deviceContext;
		/// <summary>
		/// Direct2D 设备上下文。
		/// </summary>
		private DeviceContext otherDeviceContext;
		/// <summary>
		/// DXGI SwapChain。
		/// </summary>
		private SwapChain swapChain;
		/// <summary>
		/// SwapChain 缓冲区。
		/// </summary>
		private Surface backBuffer;
		/// <summary>
		/// 渲染的目标位图。
		/// </summary>
		private Bitmap1 targetBitmap;
		/// <summary>
		/// DeviceContext 画刷。
		/// </summary>
		private Brush dcBrush;
		/// <summary>
		/// 初始化 DeviceContext。
		/// </summary>
		private void InitDeviceContext()
		{
			// 创建 Dierect3D 设备。
			D3D11Device d3DDevice = new D3D11Device(DriverType.Hardware, DeviceCreationFlags.BgraSupport);
			DXGIDevice dxgiDevice = d3DDevice.QueryInterface<D3D11Device1>().QueryInterface<DXGIDevice>();
			// 创建 Direct2D 设备和工厂。
			D2D1Device d2DDevice = new D2D1Device(dxgiDevice);
			this.deviceContext = new DeviceContext(d2DDevice, DeviceContextOptions.None);
			this.otherDeviceContext = new DeviceContext(d2DDevice, DeviceContextOptions.None);

			// 创建 DXGI SwapChain。
			SwapChainDescription swapChainDesc = new SwapChainDescription()
			{
				BufferCount = 1,
				Usage = Usage.RenderTargetOutput,
				OutputHandle = dcRenderControl.Handle,
				IsWindowed = true,
				// 这里宽度和高度都是 0，表示自动获取。
				ModeDescription = new ModeDescription(0, 0, new Rational(60, 1), Format),
				SampleDescription = new SampleDescription(1, 0),
				SwapEffect = SwapEffect.Discard
			};
			this.swapChain = new SwapChain(dxgiDevice.GetParent<Adapter>().GetParent<DXGIFactory>(),
				d3DDevice, swapChainDesc);
			// 创建 BackBuffer。
			this.backBuffer = Surface.FromSwapChain(this.swapChain, 0);
			// 从 BackBuffer 创建 DeviceContext 可用的目标。
			this.targetBitmap = new Bitmap1(this.deviceContext, backBuffer);
			this.deviceContext.Target = targetBitmap;
			// 初始化画刷资源。
			dcBrush = new SolidColorBrush(this.deviceContext, Color.Black);
		}
		/// <summary>
		/// 绘制控件的事件。
		/// </summary>
		private void dcRenderControl_Paint(object sender, PaintEventArgs e)
		{
			using (Bitmap1 bmp = new Bitmap1(this.otherDeviceContext, new Size2(200, 200), BitmapProps1))
			{
				// 使用其它 DeviceContext 绘制一幅位图。
				otherDeviceContext.Target = bmp;
				otherDeviceContext.BeginDraw();
				otherDeviceContext.Clear(Color.Transparent);
				otherDeviceContext.DrawRectangle(new RectangleF(50, 50, 100, 100), dcBrush);
				otherDeviceContext.EndDraw();

				deviceContext.BeginDraw();
				deviceContext.Clear(Color.White);
				deviceContext.DrawRectangle(new RectangleF(10, 10, 200, 200), dcBrush);
				deviceContext.DrawBitmap(bmp, 1f, BitmapInterpolationMode.Linear);
				deviceContext.EndDraw();
			}
			// 记得令 SwapChain 呈现绘制结果。
			swapChain.Present(0, PresentFlags.None);
		}
		/// <summary>
		/// 控件大小被改变的事件。
		/// </summary>
		private void dcRenderControl_SizeChanged(object sender, EventArgs e)
		{
			// 在控件大小被改变的时候必须改变渲染目标的大小，
			// 否则会导致绘制结果被拉伸，引起失真。
			// 必须先释放与 SwapChain 相关的任何资源。
			this.deviceContext.Target = null;
			this.backBuffer.Dispose();
			this.targetBitmap.Dispose();
			this.swapChain.ResizeBuffers(1, 0, 0, Format, SwapChainFlags.None);
			this.backBuffer = Surface.FromSwapChain(this.swapChain, 0);
			this.targetBitmap = new Bitmap1(this.deviceContext, backBuffer);
			this.deviceContext.Target = targetBitmap;
		}

		#endregion // DeviceContext

	}
}
